package aexp.aidlperfclient;

import android.app.Activity;
import android.os.Bundle;
import android.os.IBinder;
import android.os.DeadObjectException;
import android.widget.Button;
import android.content.Intent;
import android.content.ComponentName;
import android.content.ServiceConnection;
import android.content.Context;
import android.util.Log;
import android.view.View;
import android.widget.TextView;
import aexp.aidl.IAdderService;

public class AIDLPerfServiceClient extends Activity
{
    private IAdderService service;
    private AdderServiceConnection conn;

    class AdderServiceConnection implements ServiceConnection
    {
        public void onServiceConnected(ComponentName className, 
			IBinder boundService )
        {
          service = IAdderService.Stub.asInterface((IBinder)boundService);
		  Log.d( "ADDERSERVICECLIENT","onServiceConnected" );
        }

        public void onServiceDisconnected(ComponentName className)
        {
          service = null;
		  Log.d( "ADDERSERVICECLIENT","onServiceDisconnected" );
        }
    };

    /** Called with the activity is first created. */
    @Override
    public void onCreate(Bundle icicle)
    {
        super.onCreate(icicle);
        setContentView(R.layout.main);
		initService();
        Button invokeServiceButton = (Button) findViewById(R.id.invokeserv);
        invokeServiceButton.setOnClickListener(new View.OnClickListener() {
            public void onClick(View view) {
	          TextView t = (TextView)findViewById(R.id.invokeserv_result);
		      t.setText( "Measuring ..." );
			  long sbs = System.currentTimeMillis();
			  boolean failed = false;
			  for( int i = 0 ; i < 1000 ; ++i ) {
			    failed = invokeService( 2,3 );
				if( failed )
					break;
			  }
			  if( !failed ) {
			    long time = System.currentTimeMillis() - sbs;
			    t.setText( Long.toString( time ) + "msec" );
			  }
     		}
        });
    }

    @Override
    protected void onDestroy() {
	  releaseService();
    }

	private void initService() {
	  conn = new AdderServiceConnection();
	  Intent i = new Intent();
	  i.setClassName( "aexp.aidl", "aexp.aidl.AdderServiceImpl" );
	  bindService( i, null, conn, Context.BIND_AUTO_CREATE);
	}

	private void releaseService() {
	  unbindService( conn );	  
	  conn = null;
	}

    private boolean invokeService( int i1, int i2 ) {
	  TextView t = (TextView)findViewById(R.id.invokeserv_result);
	  if( service == null ) {
		t.setText( "Service not available" );
		return true;
	  } else {
		try {
		  int result = service.add( i1,i2 );
		  t.setText( Integer.toString( result ) );
		} catch( DeadObjectException ex ) {
		  return true;
		}
	  }
	  return false;
	}
}
